package org.jboss.seam.example.booking;


import java.security.Identity;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.faces.context.FacesContext;

import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.security.Credentials;
import org.jboss.seam.ScopeType;
import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.ConnectionConfiguration.SecurityMode;
import org.jivesoftware.smack.MessageListener;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.RosterListener;
import org.jivesoftware.smack.SmackConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.Presence.Type;

@Scope(ScopeType.SESSION)
@Name("myxmppmanager")
public class MyXMPPManager extends XMPPManager {

	    @In Identity identity;
	    @In Credentials credentials;
	    
	    @In(required=true) 
	    @Out(required=true)
	    private XMPPFriendList xmppfriendlist;
	    
		private static final int packetReplyTimeout = 500; // millis
	
	    private String server;
	    private int port;
	    private ConnectionConfiguration config;
	    private XMPPConnection connection;
	    private ChatManager chatManager;
	    private MessageListener messageListener;
	    private List<XMPPFriend> onlinefrnds;
	
	    public MyXMPPManager(){
	    	if(connection==null){
				try {
					init();
					performLogin(credentials.getUsername(),credentials.getPassword());
					getalldata();
				} catch (XMPPException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
	    	else
	    	  setOnlinefrnds(fetchusablerosterentries()); 
	    }
	    
	    public MyXMPPManager(String server, int port) {
	    	this.server = server;
	        this.port = port;
	        try {
				init();
			} catch (XMPPException e) {
				e.printStackTrace();
			}
	    }
	    
	    private Roster r;
		
		private Presence presence;
		
		public void getalldata(){
			
			setStatus(true, "In Challenge World");
			setR(fetchroster());
			setOnlinefrnds(fetchusablerosterentries());
		}
		
		public List<XMPPFriend> getOnlinefrnds() {
			 if (FacesContext.getCurrentInstance().getRenderResponse()) {
				 setOnlinefrnds(fetchusablerosterentries()); // Reload to get most recent data.
		        }
			return onlinefrnds;
		}
		public void setOnlinefrnds(List<XMPPFriend> onlinefrnds) {
			this.onlinefrnds = onlinefrnds;
		}
		
		public Roster getR() {
			return r;
		}
		public void setR(Roster r) {
			this.r = r;
		}
		public Presence getPresence() {
			return presence;
		}
		public void setPresence(Presence presence) {
			this.presence = presence;
		}
	    public List<XMPPFriend> fetchusablerosterentries(){
			
	    	List<XMPPFriend> f = new ArrayList<XMPPFriend>();
	    	XMPPFriend temp = new XMPPFriend();
	    	Roster roster = fetchroster();
	    	Collection<RosterEntry> entries = fetchrosterentries();
	    	for(RosterEntry entry : entries){
	    		String u = entry.getUser().toString();
	    		temp.setName(u);
			    Presence presence = roster.getPresence(u);
			    temp.setPresence(presence);
			    f.add(temp);
	    	}
	    	return f;
		}
	
	    
	    public void init() throws XMPPException {
	        System.out.println(String.format("Initializing connection to server %1$s port %2$d", server, port)); 
	        SmackConfiguration.setPacketReplyTimeout(packetReplyTimeout);
	        config = new ConnectionConfiguration(server, port);
	        config.setSASLAuthenticationEnabled(false);
	        config.setSecurityMode(SecurityMode.disabled);
	        connection = new XMPPConnection(config);
	        connection.connect();
	
	        System.out.println("Connected: " + connection.isConnected());
	        chatManager = connection.getChatManager();
	        messageListener = new MyMessageListener();
	    }
	
	    public void performLogin(String username, String password) throws XMPPException {
	        if (connection!=null && connection.isConnected()) {
	            connection.login(username, password);
	        }	
	    }
	
	    public void setStatus(boolean available, String status) {
	        Presence.Type type = available? Type.available: Type.unavailable;
	        Presence presence = new Presence(type);
	        presence.setStatus(status);
	        connection.sendPacket(presence);
	    }
	
	    public void destroy() {
	        if (connection!=null && connection.isConnected()) {
	            connection.disconnect();
	        }
	    }
	
	    public void sendMessage(String message, String buddyJID) throws XMPPException {
	        System.out.println(String.format("Sending mesage '%1$s' to user %2$s", message, buddyJID));
	        Chat chat = chatManager.createChat(buddyJID, messageListener);
	        chat.sendMessage(message);
	
	    }
	
	    public void createEntry(String user, String name) throws Exception {
	
	        System.out.println(String.format("Creating entry for buddy '%1$s' with name %2$s", user, name));
	        Roster roster = connection.getRoster();	
	        roster.createEntry(user, name, null);
	    }
	
	    public Collection<RosterEntry> fetchrosterentries(){
	    	
	    	Roster roster = connection.getRoster();	
	    	Collection<RosterEntry> entries = roster.getEntries();
			return entries;
	    	
	    }
	    public Roster fetchroster(){
	    	Roster roster = connection.getRoster();	
			return roster;
	    	
	    }
	    class MyMessageListener implements MessageListener {
	
	    	@Override
	        public void processMessage(Chat chat, Message message) {
	            String from = message.getFrom();
	            String body = message.getBody();
	            System.out.println(String.format("Received message '%1$s' from %2$s", body, from));

	        }
	         
	    }
	    
	    class MyRosterListener implements  RosterListener {
	    	
	    	@Override
	        public void entriesDeleted(Collection<String> addresses) {
	        	
	        }
	        @Override
	        public void entriesUpdated(Collection<String> addresses) {
	        	
	        }
	        @Override
	        public void presenceChanged(Presence presence) {
	            System.out.println("Presence changed: " + presence.getFrom() + " " + presence);
	        }
			@Override
			public void entriesAdded(Collection<String> arg0) {
				// TODO Auto-generated method stub
				
			}
	    }

	    
	    public ConnectionConfiguration getConfig() {
			return config;
		}
		public ChatManager getChatManager() {
			return chatManager;
		}
		public void setChatManager(ChatManager chatManager) {
			this.chatManager = chatManager;
		}
		public XMPPConnection getConnection() {
			return connection;
		}
		public void setConnection(XMPPConnection connection) {
			this.connection = connection;
		}
		public int getPort() {
			return port;
		}
		public void setPort(int port) {
			this.port = port;
		}
		public void setConfig(ConnectionConfiguration config) {
			this.config = config;
		}
		public String getServer() {
			return server;
		}
		public void setServer(String server) {
			this.server = server;
		}
}
